#property indicator_chart_window

#include <Ovo/winapi/MT4NamedPipeClient.class.mqh>
#include <Ovo/MT4IndicatorManager.class.mqh>

input string SERVER_NAME = "mt4pipename";  // pipe name
input int diffPoints = 100;  // Difference threshold
/*input*/ int averageTicks = 30;  // Remote ticks for price average shift

void main() {
   manager.add(new RunReceiver());
}

struct Data : MT4Structure {
   ulong localmillis;      
   double ask;
   double bid;
   ulong serial;
};

class RunReceiver: public MT4TimerRecuringListener {
   MT4NamedPipeClient* client;
   double remotePrice;
   double meanPrice;
   double diff;
   Data d;
   double offset[];
   long startMs;
   int ticksFromStart;
   int thresholdPrice;
   double koefficient;

public:
   ~RunReceiver() {
      deleteSafely(client);
      ObjectDelete("otherprice");
      ObjectDelete("meanprice");
      Comment("");
   }
   RunReceiver() :MT4TimerRecuringListener(1) {
      client = new MT4NamedPipeClient(SERVER_NAME);
      ArrayResize(offset, averageTicks);
      ArrayFill(offset, 0, averageTicks, 0.0);
      ticksFromStart = 0;
      koefficient = 0;
   }
   
   void run() {
      meanPrice = (Ask+Bid)/2;
      double delta = (meanPrice - remotePrice +diff)/Point;

    if (delta > diffPoints ||delta < -diffPoints) {
      }
      else {
         if (startMs > 0) {
            Print ("dur = " + (GetTickCount() - startMs) + " diffference = " + thresholdPrice);
            PlaySound("Alert.wav");
            startMs = 0;
            thresholdPrice = 0.0;
         } 
      Comment("");
      }

      //ObjectSet("otherprice", OBJPROP_PRICE1, (d.ask+d.bid)/2);

      if (ObjectCreate("meanprice", OBJ_TREND, 0, 0, 0)) {
         ObjectSet("meanprice", OBJPROP_WIDTH, 4);
         ObjectSet("meanprice", OBJPROP_COLOR, clrRed);               
         ObjectSet("meanprice", OBJPROP_RAY, 0);
      }
      ObjectSet("meanprice", OBJPROP_PRICE1, meanPrice);
      ObjectSet("meanprice", OBJPROP_PRICE2, meanPrice);
      ObjectSet("meanprice", OBJPROP_TIME1, Time[0]);
      ObjectSet("meanprice", OBJPROP_TIME2, (Time[0] + _Period*240) );
      
      if (!client.isOpen()) {
         client.open();
         // init read
         client.readStruct(d,sizeof(d));
      }
      
      if (client.isOpen()) {
         uint i;
         for (;client.getResult(i);client.readStruct(d,sizeof(d))) {
            if (ObjectCreate("otherprice", OBJ_TREND, 0, 0, 0)) {
               ObjectSet("otherprice", OBJPROP_WIDTH, 4);
               ObjectSet("otherprice", OBJPROP_COLOR, clrGreen);
               ObjectSet("otherprice", OBJPROP_RAY, 0);
               
            }            
            Print(GetTickCount() - d.localmillis);
            koefficient = Bid / d.bid;
            
            remotePrice = (d.ask+d.bid)*koefficient/2;

            diff = 0.0;
            int ind = d.serial%averageTicks;
            offset[ind] = remotePrice - meanPrice;
            ticksFromStart++;
            ticksFromStart = min(ticksFromStart, averageTicks);
            for (i=0; i<averageTicks; i++) {
               diff += offset[i];
            }
            diff /= ticksFromStart;

            ObjectSet("otherprice", OBJPROP_PRICE1, remotePrice-diff);
            ObjectSet("otherprice", OBJPROP_PRICE2, remotePrice-diff);
            ObjectSet("otherprice", OBJPROP_TIME2, Time[0]);
            ObjectSet("otherprice", OBJPROP_TIME1, Time[4]);
             delta = (meanPrice - remotePrice +diff)/Point;
             if (delta > diffPoints ||delta < -diffPoints) {
               Comment(/*"ask= ", d.ask, "                  \n",
                  "bid= ", d.bid, "                  \n", 
                  "serial= ", d.serial, "                  \n",*/
                  "DELTA = ", DoubleToString((meanPrice - remotePrice +diff)/Point, 0)
                  );
                  if (startMs == 0) {
                     startMs = GetTickCount();
                  }
                  if (thresholdPrice == 0) {
                     thresholdPrice = delta;
                  }
            }
            else {
            }

         }
         
      } else {
      }
   }
};

